import { execSync } from 'child_process'
import { readdirSync, readFileSync, writeFileSync } from 'fs'
import { join } from 'path'
import { config } from '../config'
import ora from 'ora'

export function build() {
  const spinner = ora('Building functions').start()
  const functions = readdirSync(config.functionsDir).filter((f) =>
    f.endsWith('.ts')
  )

  execSync(`rm -rf ${config.buildDir}`)
  execSync(`mkdir ${config.buildDir}`)
  for (const functionFile of functions) {
    const functionName = functionFile.replace('.ts', '')
    spinner.text = `Building ${functionName}`

    execSync(
      `npx tsc --module CommonJS ${join(
        config.functionsDir,
        functionFile
      )} --outDir ${join(config.buildDir, functionName)}`
    )

    writeFileSync(
      join(config.buildDir, functionName, 'package.json'),
      JSON.stringify({
        main: 'index.js',
        dependencies: JSON.parse(readFileSync('./package.json').toString())
          .dependencies,
      })
    )

    writeFileSync(
      join(config.buildDir, functionName, 'index.js'),
      `exports.${functionName} = require('./${join(
        config.functionsDir,
        functionName
      )}').${functionName};`
    )
  }

  spinner.succeed('Built functions')
}
